﻿using Bozhon.Wms.Util.Cache;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.DataProtection;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Controllers;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.AspNetCore.Mvc.Infrastructure;
using Microsoft.AspNetCore.Mvc.Routing;
using Microsoft.AspNetCore.ResponseCompression;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.DependencyInjection.Extensions;
using System;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Text.Encodings.Web;
using System.Text.Unicode;

namespace Bozhon.Wms.Api.Extensions
{
    public static class ServiceCollectionExt
    {
        /// <summary>
        /// HttpContext上下文
        /// </summary>
        /// <param name="services"></param>
        /// <returns></returns>
        public static IServiceCollection AddHttpContext(this IServiceCollection services)
        {
            //services.AddHttpContextAccessor();
            return services.AddSingleton<IHttpContextAccessor, HttpContextAccessor>();
        }

        public static IServiceCollection AddHttpClientFactory(this IServiceCollection services)
        {
            //IHttpClientFactory HttpClientFactory
            return services.AddHttpClient();
        }

        public static IServiceCollection AddContextAccessor(this IServiceCollection services)
        {
            if (services.Count(x => x.ServiceType == typeof(IHttpContextAccessor)) == 0)
                services.AddSingleton<IHttpContextAccessor, HttpContextAccessor>();
            if (services.Count(x => x.ServiceType == typeof(IActionContextAccessor)) == 0)
                services.AddSingleton<IActionContextAccessor, ActionContextAccessor>();
            return services;
        }

        //public static IServiceCollection AddContextFactory(this IServiceCollection services)
        //{
        //    //return services.AddSingleton<IHttpContextFactory, HttpContextFactory>();
        //}

        /// <summary>
        /// url小写
        /// </summary>
        /// <param name="services"></param>
        /// <returns></returns>
        public static IServiceCollection AddLowercaseUrls(this IServiceCollection services)
        {
            services.AddRouting(options =>
            {
                options.LowercaseUrls = true;
            });
            return services;
        }

        public static IServiceCollection AddUrlHelper(this IServiceCollection services)
        {
            services
                .AddSingleton<IActionContextAccessor, ActionContextAccessor>()
                .AddScoped(it =>
                    it
                        .GetRequiredService<IUrlHelperFactory>()
                        .GetUrlHelper(it.GetRequiredService<IActionContextAccessor>().ActionContext));
            return services;
        }

        /// <summary>
        /// Configuration
        /// </summary>
        /// <param name="services"></param>
        /// <param name="configuration"></param>
        /// <returns></returns>
        public static IServiceCollection AddConfiguration(this IServiceCollection services, IConfigurationRoot configuration)
        {
            return services.AddSingleton(configuration);
        }

        /// <summary>
        /// 中文乱码
        /// </summary>
        /// <param name="services"></param>
        /// <returns></returns>
        public static IServiceCollection AddHtmlEncoder(this IServiceCollection services)
        {
            return services.AddSingleton(HtmlEncoder.Create(UnicodeRanges.All));
        }

     

      

        /// <summary>
        ///Br
        /// </summary>
        /// <param name="services"></param>
        /// <returns></returns>
        public static IServiceCollection AddBr(this IServiceCollection services)
        {
            return services.AddResponseCompression(options =>
            {
                options.Providers.Add<BrotliCompressionProvider>();
                options.MimeTypes = ResponseCompressionDefaults.MimeTypes.Concat(new[] { "image/svg+xml" });
            });
        }

        /// <summary>
        /// Gzip
        /// </summary>
        /// <param name="services"></param>
        /// <returns></returns>
        public static IServiceCollection AddGzip(this IServiceCollection services)
        {
            return services.AddResponseCompression(options =>
            {
                options.Providers.Add<GzipCompressionProvider>();
                options.MimeTypes = ResponseCompressionDefaults.MimeTypes.Concat(new[]
                {
                     // Default
                    "text/plain",
                    "text/css",
                    "application/javascript",
                    "text/html",
                    "application/xml",
                    "text/xml",
                    "application/json",
                    "text/json",
                    // Custom
                    "image/svg+xml",
                    "font/woff2",
                    "application/font-woff",
                    "application/font-ttf",
                    "application/font-eot",
                    "image/jpeg",
                    "image/png"
                 });
            }).Configure<GzipCompressionProviderOptions>(options =>
            {
                options.Level = CompressionLevel.Fastest;
            });
        }

        public static IServiceCollection AddXsrf(this IServiceCollection services)
        {
            return services.AddAntiforgery(options =>
            {
                //X-CSRF-TOKEN
                options.HeaderName = "X-XSRF-TOKEN";
            });
        }

    

        /// <summary>
        /// 上传大小限制
        /// </summary>
        /// <param name="services"></param>
        /// <param name="key"></param>
        /// <param name="configuration"></param>
        /// <returns></returns>
        //public static IServiceCollection AddMultipartBodyLengthLimit(this IServiceCollection services, IConfiguration configuration, string key = null)
        //{
        //    if (key.IsEmpty())
        //    {
        //        //throw new ArgumentNullException(nameof(key));
        //        services.Configure<FormOptions>(
        //        f =>
        //        {
        //            f.ValueLengthLimit = int.MaxValue;
        //            f.MultipartBodyLengthLimit = int.MaxValue;
        //        }
        //        );
        //    }
        //    else
        //    {
        //        services.Configure<FormOptions>(
        //       f =>
        //       {
        //           f.ValueLengthLimit = configuration[key].ToInt32();
        //           f.MultipartBodyLengthLimit = configuration[key].ToInt32();
        //       }
        //       );
        //    }
        //    return services;
        //}

        /// <summary>
        /// 数据保护
        /// </summary>
        /// <param name="services"></param>
        /// <returns></returns>
        public static IServiceCollection AddDataProtectionL(this IServiceCollection services)
        {
            services.AddDataProtection().PersistKeysToFileSystem(new DirectoryInfo(Directory.GetCurrentDirectory()));
            return services;
        }

      

        public static IServiceCollection AddCSRedisCache(this IServiceCollection services, IConfiguration configuration)
        {
            services.AddScoped<IRedisClient, RedisUtil>();
            var csredis = new CSRedis.CSRedisClient(configuration["Redis"]);
            RedisHelper.Initialization(csredis);
            return services;
        }

    
      
        public static IServiceCollection AddLazy(this IServiceCollection services, ServiceLifetime lifetime = ServiceLifetime.Transient)
        {
            switch (lifetime)
            {
                case ServiceLifetime.Singleton:
                    return services.AddSingleton(typeof(Lazy<>));

                case ServiceLifetime.Scoped:
                    return services.AddScoped(typeof(Lazy<>));

                case ServiceLifetime.Transient:
                    return services.AddTransient(typeof(Lazy<>));

                default:
                    return services.AddTransient(typeof(Lazy<>));
            }
        }

        //public static IServiceCollection AddDIProperty(this IServiceCollection services)
        //{
        //    //替换控制器构建者类，实现控制器和Filter类属性注入功能
        //    services.Replace(ServiceDescriptor.Transient<IControllerActivator, PropertiesAutowiredControllerActivator>());
        //    services.Replace(ServiceDescriptor.Transient<IFilterProvider, PropertiesAutowiredFilterProvider>());
        //    return services;
        //}
    }
}
